/*
  HUDless.c - Remove the HUD.

  Jason Hood, 30 & 31 October, 2017.

  Create a secondary "MINIMIZE HUD" key, using a modifier (one or more of
  Shift, Ctrl or Alt), to remove the display of the HUD, then the cursor, then
  restore both.

  Install:
    add an entry for HUDless.dll to [Libraries] in EXE\dacom.ini.
*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define NAKED	__declspec( naked )
#define STDCALL __stdcall


#define ADDR_MINHUD 0x4d5383


DWORD dummy;
#define ProtectX( addr, size ) \
  VirtualProtect( (PVOID)(addr), size, PAGE_EXECUTE_READWRITE, &dummy )

#define RELOFS( from, to ) \
  *(PDWORD)(from) = (DWORD)(to) - (DWORD)(from) - 4;

#define JMP( from, to ) \
  *(PBYTE)(from) = 0xE9; \
  RELOFS( (DWORD)from+1, to )


NAKED
void MinHUD_Hook( void )
{
  __asm {
	cmp	ds:[0x67c280], al	// any modifier keys pressed?
	jz	done			// no, normal routine
	cmp	ds:[0x679c0c], al
	jnz	HUD
	cmp	ds:[0x679c20], al
	sete	al
	mov	ds:[0x679c20], al	// cursor
  HUD:
	mov	ds:[0x679c0c], al	// the icons
	mov	ds:[0x679c10], al	// everything else
	mov	ds:[0x679c40], al	// scroll bars on weapon list
	mov	al, 1
	ret	4
  done:
	mov	eax, 0x4d538a
	jmp	eax
	align	16
  }
}


void Patch( void )
{
  ProtectX( ADDR_MINHUD, 7 );

  *(WORD*)ADDR_MINHUD = 0xF475;
  JMP( ADDR_MINHUD + 2, MinHUD_Hook );
}


BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
{
  if (fdwReason == DLL_PROCESS_ATTACH)
    Patch();

  return TRUE;
}
